import * as TE from "fp-ts/lib/TaskEither";
import * as O from "fp-ts/lib/Option";
import type { TestJob } from "./job";
import type { D } from "../util";

export enum TestType {
  EMAIL = "email",
  PING = "ping",
  HEALTHCHECK = "healthcheck",
  DNS = "dns"
}

export interface Schedule {
  every: D.Duration;
  jitter: D.Duration;
}

export interface Test {
  name: string;
  type: TestType;
  job: TestJob;
  schedule: Schedule;
}

export type Testable<TJob extends TestJob> = (job: TJob) => TE.TaskEither<Error, boolean>;

export const parseTestType = (testType: string): O.Option<TestType> =>
  Object.values(TestType).includes(testType as TestType) ? O.some(testType as TestType) : O.none;

export const nextSchedule = (schedule: Schedule, date: Date) =>
  new Date(date.getTime() + schedule.every + Math.random() * schedule.jitter);
