import { toError } from "fp-ts/lib/Either";
import * as TE from "fp-ts/TaskEither";
import { publishDiscord, type DiscordPost } from "./discord";
import { publishNtfy, type NtfyPost } from "./ntfy";

export enum PublisherType {
  DISCORD = "discord",
  NTFY = "ntfy"
}
export type PublisherPost = DiscordPost | NtfyPost;

export interface Publisher {
  type: PublisherType;
  post: PublisherPost;
}

export const publish = (publisher: Publisher, message: string): TE.TaskEither<Error, number> => {
  switch (publisher.type) {
    case PublisherType.DISCORD:
      return TE.tryCatch(() => publishDiscord(publisher.post as DiscordPost, message), toError);
    case PublisherType.NTFY:
      return TE.tryCatch(() => publishNtfy(publisher.post as NtfyPost, message), toError);
    default:
      return TE.left(new Error("unknown publisher type: " + publisher.type));
  }
};
